@@ -33,6 +33,14 @@ typedef struct {
     AMsyncState *state;     // Borrowed pointer extracted from result
 } am_syncstate;
 
+// Change wrapper (owned changes only)
+// Stores the owning AMresult* and the borrowed AMchange* pointer.
+// Borrowed changes store AMchange* directly in the ext_ptr (no struct).
+typedef struct {
+    AMresult *result;   // Owns the change (freed in finalizer)
+    AMchange *change;   // Borrowed pointer extracted from result
+} am_change_data;
+
 // Function Declarations -------------------------------------------------------
 
 // Document operations (document.c)
@@ -50,6 +58,28 @@ SEXP C_am_rollback(SEXP doc_ptr);
 SEXP C_am_get_last_local_change(SEXP doc_ptr);
 SEXP C_am_get_change_by_hash(SEXP doc_ptr, SEXP hash);
 SEXP C_am_get_changes_added(SEXP doc1_ptr, SEXP doc2_ptr);
+// Change introspection (changes.c)
+SEXP C_am_change_hash(SEXP change);
+SEXP C_am_change_message(SEXP change);
+SEXP C_am_change_time(SEXP change);
+SEXP C_am_change_actor_id(SEXP change);
+SEXP C_am_change_seq(SEXP change);
+SEXP C_am_change_deps(SEXP change);
+SEXP C_am_change_from_bytes(SEXP bytes);
+SEXP C_am_change_to_bytes(SEXP change);
+SEXP C_am_change_size(SEXP change);
+
+// v1.2 Document operations (document.c)
+SEXP C_am_clone(SEXP doc_ptr);
+SEXP C_am_equal(SEXP doc1_ptr, SEXP doc2_ptr);
+SEXP C_am_pending_ops(SEXP doc_ptr);
+SEXP C_am_commit_empty(SEXP doc_ptr, SEXP message, SEXP time);
+SEXP C_am_save_incremental(SEXP doc_ptr);
+SEXP C_am_load_incremental(SEXP doc_ptr, SEXP data);
+
+// Shared helpers
+AMresult* convert_r_heads_to_amresult(SEXP heads_list, AMresult ***results_out, size_t *n_results);
+AMitems* resolve_heads(SEXP heads, AMitems *heads_items_out, AMresult **heads_result_out);
 
 // Object operations (objects.c)
 SEXP C_am_put(SEXP doc_ptr, SEXP obj_ptr, SEXP key_or_pos, SEXP value);
@@ -63,29 +93,54 @@ SEXP C_am_text_content(SEXP text_ptr);
 SEXP C_am_text_update(SEXP text_ptr, SEXP old_str, SEXP new_str);
 SEXP C_am_values(SEXP doc_ptr, SEXP obj_ptr);
 SEXP C_am_counter_increment(SEXP doc_ptr, SEXP obj_ptr, SEXP key_or_pos, SEXP delta);
+// v1.2 Object operations (objects.c)
+SEXP C_am_map_get_all(SEXP doc_ptr, SEXP obj_ptr, SEXP key, SEXP heads);
+SEXP C_am_list_get_all(SEXP doc_ptr, SEXP obj_ptr, SEXP pos, SEXP heads);
+SEXP C_am_map_range(SEXP doc_ptr, SEXP obj_ptr, SEXP begin, SEXP end, SEXP heads);
+SEXP C_am_list_range(SEXP doc_ptr, SEXP obj_ptr, SEXP begin, SEXP end, SEXP heads);
+SEXP C_am_items(SEXP doc_ptr, SEXP obj_ptr, SEXP heads);
 
 // Synchronization operations (sync.c)
-SEXP C_am_sync_state_new(void);
+SEXP C_am_sync_state(void);
 SEXP C_am_sync_encode(SEXP doc_ptr, SEXP sync_state_ptr);
 SEXP C_am_sync_decode(SEXP doc_ptr, SEXP sync_state_ptr, SEXP message);
 SEXP C_am_get_heads(SEXP doc_ptr);
 SEXP C_am_get_changes(SEXP doc_ptr, SEXP heads);
 SEXP C_am_apply_changes(SEXP doc_ptr, SEXP changes);
+// v1.2 Sync/change operations (sync.c, changes.c)
+SEXP C_am_get_missing_deps(SEXP doc_ptr, SEXP heads);
+SEXP C_am_load_changes(SEXP data);
+SEXP C_am_sync_state_encode(SEXP sync_state_ptr);
+SEXP C_am_sync_state_decode(SEXP data);
 
 // Cursor and mark operations (cursors.c)
-SEXP C_am_cursor(SEXP obj_ptr, SEXP position);
-SEXP C_am_cursor_position(SEXP cursor_ptr);
+SEXP C_am_cursor(SEXP obj_ptr, SEXP position, SEXP heads);
+SEXP C_am_cursor_position(SEXP cursor_ptr, SEXP heads);
 SEXP C_am_mark(SEXP obj_ptr, SEXP start, SEXP end, SEXP name, SEXP value, SEXP expand);
-SEXP C_am_marks(SEXP obj_ptr);
-SEXP C_am_marks_at(SEXP obj_ptr, SEXP position);
+SEXP C_am_marks(SEXP obj_ptr, SEXP heads);
+SEXP C_am_marks_at(SEXP obj_ptr, SEXP position, SEXP heads);
+SEXP C_am_cursor_to_bytes(SEXP cursor_ptr);
+SEXP C_am_cursor_from_bytes(SEXP bytes, SEXP obj_ptr);
+SEXP C_am_cursor_to_string(SEXP cursor_ptr);
+SEXP C_am_cursor_from_string(SEXP str, SEXP obj_ptr);
+SEXP C_am_cursor_equal(SEXP cursor1_ptr, SEXP cursor2_ptr);
+// v1.2 Mark operations (cursors.c)
+SEXP C_am_mark_clear(SEXP obj_ptr, SEXP start, SEXP end, SEXP name, SEXP expand);
 
 // Finalizers (memory.c)
 void am_doc_finalizer(SEXP ext_ptr);
 void am_result_finalizer(SEXP ext_ptr);
 void am_syncstate_finalizer(SEXP ext_ptr);
+void am_change_finalizer(SEXP ext_ptr);
+
+// Change wrapping helpers (changes.c)
+SEXP wrap_am_change_owned(AMresult *result);
+SEXP wrap_am_change_borrowed(AMchange *ch, SEXP parent_result_ptr);
+AMchange *get_change(SEXP change_ptr);
 
 // Helper functions (memory.c)
-AMdoc *get_doc(SEXP doc_ptr);  // Returns borrowed AMdoc* pointer
+AMdoc *get_doc(SEXP doc_ptr);
+AMsyncState *get_syncstate(SEXP sync_state_ptr);
 const AMobjId *get_objid(SEXP obj_ptr);
 SEXP get_doc_from_objid(SEXP obj_ptr);  // Extract doc from am_object protection chain
 SEXP C_get_doc_from_objid(SEXP obj_ptr);  // Exported for R .Call() interface
